import process from "node:process";
import * as docker from "./docker.js";
import * as hosts from "./hosts.js";
import { dockerNetwork } from "./const.js";

let running = true;
const stopRunning = () => {
  running = false;
};

// Signal handlers to terminate min loop
process.on("SIGTERM", stopRunning);
process.on("SIGINT", stopRunning);

hosts.init();
docker.update().then(() => {
  hosts.writeConfig();
  console.log("Startup Done");
  idleMain();
});

async function idleMain() {
  // Main Idle Loop
  // Checks Docker for changes every ten seconds
  // SIGTERM and SIGINT signals can be processed during the timeout so the process can exit gracefully
  let idleCount = 0;
  const proxyNetworkID = docker.networks[dockerNetwork].Id;

  while (running) {
    if (idleCount === 0) {
      await docker.update();
      hosts.update(Object.values(docker.containers), proxyNetworkID);
      if (hosts.writeConfig()) {
        //update caddy
      }
    }
    await new Promise((res) => setTimeout(res, 1000));
    idleCount = (idleCount + 1) % 10;
  }

  // Cleanup Code goes here (docker shutdown)
  await docker.stopCaddy();
}
