export type NetType = "Bridge" | "Host" | "none" | "null";

export interface NetworkData {
  Id: string;
  Name: string;
  Driver: NetType;
}

export interface ContainerData {
  Id: string;
  Name: string;
  State: "exited" | "running";
  Networks: Record<string, ContainerNetwork>;
  HostNames?: string[];
  Ports: string[];
  Labels: {
    HostName?: string;
    PortMap?: string[];
  };
}

export type ContainerNetwork = {
  NetworkID: string;
  IPAddress: string;
};

export type ContainerOverride = {
  data: {
    Id: string;
    Names: string[];
    State: "exited" | "running";
    NetworkSettings: {
      Networks: Record<string, ContainerNetwork>;
    };
    Labels: Record<string, string>;
    Ports: {
      IP?: string;
      PrivatePort: number;
      PublicPort?: number;
      Type: "tcp" | "udp";
    }[];
  };
};

export type NetworkOverride = {
  data: {
    Name: string;
    Id: string;
    Driver: NetType;
    Scope: string;
  };
};

export type ImageOverride = {
  data: {
    Containers: number;
    Created: number;
    Id: string;
    Labels: null | Record<string, string>;
    ParentId: string;
    RepoDigests: string[];
    RepoTags: string[];
    SharedSize: number;
    Size: number;
    VirtualSize: number;
  };
};

export type CaddyState =
  | "running"
  | "exited"
  | "nocontainer"
  | "noimage"
  | "wrongnetwork";

export interface CaddyHttpServer {
  listen?: string[];
  // "listener_wrappers"?: [{•••}],
  read_timeout?: number;
  read_header_timeout?: number;
  write_timeout?: number;
  idle_timeout?: number;
  keepalive_interval?: number;
  max_header_bytes?: number;
  routes?: {
    group?: string;
    // "match"?: [{••• }],
    // "handle"?: [{••• }],
    terminal?: boolean;
  }[];
  errors?: {
    routes?: [
      {
        group?: string;
        // "match"?: [{••• }],
        // "handle"?: [{••• }],
        terminal?: boolean;
      }
    ];
  };
  tls_connection_policies?: [
    {
      // "match"?: {••• },
      certificate_selection?: {
        serial_number?: [{}];
        subject_organization?: string[];
        public_key_algorithm?: number;
        any_tag?: string[];
        all_tags?: string[];
      };
      cipher_suites?: string[];
      curves?: string[];
      alpn?: string[];
      protocol_min?: string;
      protocol_max?: string;
      client_authentication?: {
        trusted_ca_certs?: string[];
        trusted_ca_certs_pem_files?: string[];
        trusted_leaf_certs?: string;
        // "verifiers"?: [{••• }],
        mode?: string;
      };
      default_sni?: string;
      insecure_secrets_log?: string;
    }
  ];
  automatic_https?: {
    disable?: boolean;
    disable_redirects?: boolean;
    disable_certificates?: boolean;
    skip?: string[];
    skip_certificates?: string[];
    ignore_loaded_certificates?: boolean;
  };
  strict_sni_host?: boolean;
  logs?: {
    default_logger_name?: string;
    logger_names?: Record<string, string>;
    skip_hosts?: string[];
    skip_unmapped_hosts?: boolean;
    should_log_credentials?: boolean;
  };
  protocols?: string[];
  metrics?: {};
}

export interface CaddyHttp {
  http_port: number;
  https_port?: number;
  grace_period?: number;
  shutdown_delay?: number;
  servers: Record<string, CaddyHttpServer>;
}

export interface CaddyFileSystem {
  module: "file_system";
  root: string;
}

export interface Caddy {
  admin?: {
    /** If true, the admin endpoint will be completely disabled. */
    disabled?: boolean;
    /** The address to which the admin endpoint's listener should bind itself. */
    listen?: string;
    enforce_origin?: boolean;
    origins?: string[];
    config?: {
      persist?: boolean;
      // "load"?: {••• }
    };
    identity?: {
      identifiers?: string[];
      // "issuers"?: [{••• }]
    };
    remote?: {
      listen?: string;
      access_control?: [
        {
          public_keys?: string[];
          permissions?: [
            {
              paths?: string[];
              methods?: string[];
            }
          ];
        }
      ];
    };
  };
  logging?: {
    sink?: {
      // writer?: { }
    };
    logs?: Record<
      string,
      {
        // "writer"?: { },
        // "encoder"?: { },
        level?: string;
        sampling?: {
          interval?: number;
          first?: number;
          thereafter?: number;
        };
        include?: string[];
        exclude?: string[];
      }
    >;
  };
  storage?: {
    file_system?: CaddyFileSystem;
  };
  apps?: {
    http?: CaddyHttp;
  };
}

export type HostData = Record<string, string[]>;
